#include <stdlib.h>
#include "NdbApi.hpp"
#include <stdio.h>

#define APIERROR(error) { \
  printf("%d - %s \n", error.code,error.message); \
exit(-1); }


int main(int argc, char**argv)
{
  const char *connectstring = argv[1];

  ndb_init();

  // Connection
  Ndb_cluster_connection *conn;
  
  // my_instance()
  conn = new Ndb_cluster_connection(connectstring); 
  conn->set_name("bug 45282 test script");
  if(conn->connect(2,1,0))  {
    printf("Unable to connect to cluster");
    exit(-1);
  }
  if (conn->wait_until_ready(5,5) < 0) {
    printf("Cluster was not ready within 5 secs.\n");
    exit(-1);
  }
          
  Ndb *myNdb = new Ndb(conn);      
  myNdb->init();  
  printf("Connected to cluster.\n\n");


  // From Query() ... 

  //  -> Query(request_rec*, config::dir*, ndb_instance*, query_source&) 
  //    -> Ndb::setDatabaseName(char const*)   
  myNdb->setDatabaseName("test_db");

  //    -> Ndb::getDictionary() const          
  const NdbDictionary::Dictionary *myDict= myNdb->getDictionary();

  //    -> NdbDictionary::Dictionary::getTable(char const*) const 
  const NdbDictionary::Table *myTable= myDict->getTable("typ8");
  if (myTable == NULL)
    APIERROR(myDict->getNdbError());


  menu:
  
  int menu_choice;
  int bug_mode;
  
  printf("\n\n");
  printf("Enter selection:\n");
  printf("    1.   Run with no bug.\n");
  printf("    2.   Run with bug.\n");
  printf("    3.   Quit\n");
  printf("> ");
  scanf("%d", &menu_choice);
  
  if(menu_choice == 1) bug_mode = 0;
  else if(menu_choice == 2) bug_mode = 1;
  else if(menu_choice == 3) exit(0);
  else goto menu;
  

  // Line 347.  Start a Transaction  
  //   -> Ndb::startTransaction(NdbDictionary::Table const*, char const*, unsigned int) 
  NdbTransaction *myTrans= myNdb->startTransaction();
  if (myTrans == NULL)
    APIERROR(myNdb->getNdbError());


  //  -> PK_index_object::get_ndb_operation(NdbTransaction*) 
  //    -> NdbDictionary::Table::getNoOfPrimaryKeys() const 
  int nPKparts;
  nPKparts = myTable->getNoOfPrimaryKeys();

  // Start an operation  
  //  -> NdbTransaction::getNdbOperation(NdbDictionary::Table const*) 
  NdbOperation *myNdbOperation= myTrans->getNdbOperation(myTable);
  if (myNdbOperation == NULL)
    APIERROR(myTrans->getNdbError());

  //  -> Plan::SetupRead(request_rec*, config::dir*, QueryItems*) 
  //    -> NdbOperation::readTuple(NdbOperation::LockMode) 
  myNdbOperation->readTuple(NdbOperation::LM_CommittedRead);


  // Query() line 433 -- call into index_object::get_column()
  // index_object.h line 64
  // Get a column object for the one-part PK

   const NdbDictionary::Column *pk_col = 
      myTable->getColumn(myTable->getPrimaryKey(0));
    int pk_col_no = pk_col->getColumnNo();

  
  // Query() line 440 -- call into PK_index_object::set_key_part() 
  //   -> PK_index_object::set_key_part(int, mvalue&) 
  //     -> index_object::set_key_num(int, mvalue&) 
  //       -> NdbOperation::equal(unsigned int, char const*)     
  myNdbOperation->equal(pk_col_no, 1);
  
  // Query() line 511, op_action() 
  //  -> Plan::Read(request_rec*, config::dir*, QueryItems*) 
  //   -> NdbDictionary::Table::getColumn(char const*) const 
  //   -> NdbOperation::getValue(NdbDictionary::Column const*, char*) 

  const NdbDictionary::Column *blob_col = myTable->getColumn("doc");

  // line 555:
  NdbBlob ** blobs = (NdbBlob **) calloc(1, sizeof(NdbBlob *));
  

  NdbRecAttr *recattr;
  if(bug_mode) {
    // HERE IS THE PROBLEM
    recattr = myNdbOperation->getValue(blob_col, 0);
  }
  
  if((blob_col->getType() == NdbDictionary::Column::Blob) ||
     (blob_col->getType() == NdbDictionary::Column::Text)) {
    
    int inline_size = blob_col->getInlineSize();
    if(inline_size) {
      blobs[0] = myNdbOperation->getBlobHandle("doc");
    }
    printf("getInlineSize() is %d\n", inline_size);
  }
  else printf("getType() is not Blob or Text\n");
  

  // Execute "NoCommit" before reading the BLOB
  if(myTrans->execute(NdbTransaction::NoCommit))
    APIERROR(myTrans->getNdbError());
  
  // Get the blob size
  unsigned long long size64 = 0;  
  blobs[0]->getLength(size64);  //passed by reference
  
  // get a buffer
  unsigned int size = (unsigned int) size64;
  printf("buffer size: %d \n\n",size);  
  void * my_buffer = calloc(1, size); 
  
  // read the blob   
  if(blobs[0]->readData(my_buffer, size)) 
    APIERROR(blobs[0]->getNdbError());
  
  // Execute "Commit"                    
  if (-1 == myTrans->execute(NdbTransaction::Commit))
    APIERROR(myTrans->getNdbError());
  myNdb->closeTransaction(myTrans);
  
  printf("%s \n", (const char *) (my_buffer));
  
  goto menu;
}
